@@ -25,7 +25,7 @@
 
 typedef struct _MetaTilePreview MetaTilePreview;
 
-MetaTilePreview   *meta_tile_preview_new    (gboolean           composited);
+MetaTilePreview   *meta_tile_preview_new    (void);
 void               meta_tile_preview_free   (MetaTilePreview   *preview);
 void               meta_tile_preview_show   (MetaTilePreview   *preview,
                                              MetaRectangle     *rect);
