@@ -38,6 +38,8 @@ gboolean meta_window_is_shaded (MetaWindow *window);
 MetaScreen *meta_window_get_screen (MetaWindow *window);
 MetaDisplay *meta_window_get_display (MetaWindow *window);
 Window meta_window_get_xwindow (MetaWindow *window);
+Window meta_window_get_toplevel_xwindow (MetaWindow *window);
+Visual *meta_window_get_toplevel_xvisual (MetaWindow *window);
 MetaWindow *meta_window_get_transient_for (MetaWindow *window);
 gboolean meta_window_is_fullscreen (MetaWindow *window);
 gboolean meta_window_is_maximized (MetaWindow *window);
